//+------------------------------------------------------------------+
//|                                                    TickPulse.mq4 |
//|                                                           Dagnar |
//|                                                                  |
//+------------------------------------------------------------------+
#property copyright "Dagnar"
#property link      ""


#property indicator_separate_window
#property indicator_buffers 3
#property indicator_color1 Lime
#property indicator_color2 Red
#property indicator_color3 DarkTurquoise


//---- input parameters
extern int       hst=200;      //  -   
extern int       MAper=10;     //  . 
extern bool UseSimpleMA=true;  //   
extern bool UseHMA=false;      //  HMA
extern bool ShowStat=true;     //    
extern int  StatPeriod=30;     //   , 
extern int  TickTreshold=20;   //       (large ticks),  
extern int  DirCount=5;        //  - ""       
extern int  ActivityCount=10;  //   -       

//---- buffers
double ExtMapBuffer1[];
double ExtMapBuffer2[];
double ExtMapBuffer3[];
double ma_buf[];
double TickTime[];
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
  {
  
//---- indicators
   IndicatorBuffers(5);
   SetIndexStyle(0,DRAW_LINE,STYLE_SOLID,2);
   SetIndexBuffer(0,ExtMapBuffer1);
   SetIndexStyle(1,DRAW_LINE,STYLE_SOLID,2);
   SetIndexBuffer(1,ExtMapBuffer2);
   SetIndexStyle(2,DRAW_LINE);
   SetIndexBuffer(2,ExtMapBuffer3);
   SetIndexBuffer(3,ma_buf);
   SetIndexBuffer(4,TickTime);
   for (int i=0; i<hst; i++) {
   ExtMapBuffer1[i]=Close[0];
   ExtMapBuffer2[i]=Close[0];
   ExtMapBuffer3[i]=Close[0];
   ma_buf[i]=Close[0];
   TickTime[i]=TimeCurrent();
   }
   
   
   //----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//----
   
//----
   return(0);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
double WMA(int x, int p)
  {
   return(iMAOnArray(ExtMapBuffer3, 0, p, 0, 3, x));
  }
int start()
  {
   int    counted_bars=IndicatorCounted();
//----
   int i,j;
  
   for (i=hst; i>0; i--) {
    ExtMapBuffer1[i]=ExtMapBuffer1[i-1];
    ExtMapBuffer2[i]=ExtMapBuffer2[i-1];
    ExtMapBuffer3[i]=ExtMapBuffer3[i-1];
    ma_buf[i]=ma_buf[i-1];
    TickTime[i]=TickTime[i-1];
   }
 ExtMapBuffer3[0]=Close[0];
 TickTime[0]=TimeCurrent();
  double m1=0,m2=0,c1=0,c2=0;  
 if (UseSimpleMA) {

    for (j=0; j<MAper; j++) {  
     if (ExtMapBuffer3[j]!=EMPTY_VALUE) {m1=m1+ExtMapBuffer3[j];c1++;}
     if (ExtMapBuffer3[j+1]!=EMPTY_VALUE) {m2=m2+ExtMapBuffer3[j+1];c2++;}
    }
    m1=m1/c1; m2=m2/c2;
    ma_buf[0]=m1;
 } else {
     
   double ma=0,c=0;
   double vect[];
   ArrayResize(vect, hst);
   ArraySetAsSeries(vect, true);  
   for(int x=0; x < hst; x++)
     {
      vect[x]=2*WMA(x, MAper/2) - WMA(x, MAper);
      }  
   int p=MathSqrt(MAper);
   ma_buf[0]=iMAOnArray(vect, 0, p, 0, 3, 0);
   for (j=0; j<5; j++) {  
     if (ExtMapBuffer3[j]!=EMPTY_VALUE) {m1=m1+ExtMapBuffer3[j];c1++;}
     if (ExtMapBuffer3[j+1]!=EMPTY_VALUE) {m2=m2+ExtMapBuffer3[j+1];c2++;}
    }
    m1=m1/c1; m2=m2/c2;
 }  
    
    
   if (ma_buf[1]!=0 && ma_buf[1]!=EMPTY_VALUE) {
     if (ma_buf[0]>=ma_buf[1]) {ExtMapBuffer1[0]=m1;  ExtMapBuffer2[0]=EMPTY_VALUE;
       if (ma_buf[1]<ma_buf[2]) {ExtMapBuffer1[1]=m2;}
       }
     if (ma_buf[0]<ma_buf[1]) {ExtMapBuffer2[0]=m1;  ExtMapBuffer1[0]=EMPTY_VALUE;
       if (ma_buf[1]>=ma_buf[2]) {ExtMapBuffer2[1]=m2; }
     }     
   }
   int wh;
   if (!ShowStat) {
     if (ObjectFind("stat1")!=-1) {
      wh=WindowFind("TickPulse_v2");
      ObjectsDeleteAll(wh);
     }
   }
   if (ShowStat) {
     int up=0,down=0,cnt=0;
     for (i=0; i<hst-1; i++) {
       if (TimeCurrent()-TickTime[i]<=StatPeriod && ExtMapBuffer3[i]-ExtMapBuffer3[i+1]!=0) {
        cnt++;
        if (ExtMapBuffer3[i]-ExtMapBuffer3[i+1]>(TickTreshold-1)*Point) up++;
        if (ExtMapBuffer3[i]-ExtMapBuffer3[i+1]<-(TickTreshold-1)*Point) down++;
       }
     }
   wh=WindowFind("TickPulse_v2");
   int br=WindowFirstVisibleBar();
   if (ObjectFind("stat1")==-1) {
     ObjectCreate("stat1",OBJ_LABEL,wh,0,0);     
     ObjectCreate("stat2",OBJ_LABEL,wh,0,0);
     ObjectCreate("stat3",OBJ_LABEL,wh,0,0);
    }
    ObjectSet("stat1",OBJPROP_CORNER,0);
    ObjectSet("stat2",OBJPROP_CORNER,0);
    ObjectSet("stat3",OBJPROP_CORNER,0);
    
    ObjectSet("stat1",OBJPROP_YDISTANCE,15);
    ObjectSet("stat2",OBJPROP_YDISTANCE,30);
    ObjectSet("stat3",OBJPROP_YDISTANCE,45);
    
    string s1="Total ticks in "+DoubleToStr(StatPeriod,0)+"sec = "+DoubleToStr(cnt,0);
    string s2="Large ticks up = "+DoubleToStr(up,0);
    string s3="Large ticks down = "+DoubleToStr(down,0);
    if (up>=DirCount && down<up) s2=s2+"  | TREND UP !";
    if (down>=DirCount  && down>up) s3=s3+"  | TREND DOWN !";
    if (up+down<DirCount && cnt<ActivityCount) s1=s1+"  | low activity";
    
    ObjectSetText("stat1",s1,10,"Times New Roman",Yellow);
    ObjectSetText("stat2",s2,10,"Times New Roman",Lime);
    ObjectSetText("stat3",s3,10,"Times New Roman",Red);
    
   }
 
//----
   return(0);
  }
//+------------------------------------------------------------------+